<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 09/20/2005 *
 * ===================                          Last change: 09/20/2005 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : security_functions.php                           *
 * -------------------------------------------------------------------- *
 * Short description : Secure all GET, POST and COOKIE data             *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Alle GET, POST und COOKIE-Daten sichern          *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2011 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Run only once this security check/replacement
if (defined('__SECURITY')) {
	return;
} // END - if

// Some security stuff...
if (strpos($_SERVER['PHP_SELF'], basename(__FILE__)) !== false) {
	die();
} // END - if

// Include ctracker, recommended place!
//require_once('ctracker.php');

/**
 * Function to secure input strings
 *
 * @param	$str	The unsecured string
 * @param	$strip	Strip tags
 * @return	$str	A (hopefully) secured string against XSS and other bad things
 */
function secureString ($str, $strip = true, $encode = false) {
	// Shall we strip HTML code?
	if ($strip === true) $str = strip_tags($str);

	// Trim string
	$str = trim($str);

	// Encode in entities if requested
	if ($encode === true) {
		// Encode in entities (this breakes UTF-8!)
		$str = htmlentities($str, ENT_QUOTES);
	} // END - if

	// Return result
	return $str;
}

/**
 * Secures $_SERVER['PHP_SELF'] against attacks
 *
 * @return	void
 */
function securePhpSelf () {
	// Did it run before?
	if (isset($GLOBALS['php_self_secured'])) {
		// Please do not call this twice!
		die('PHP_SELF is already secured. Please do not call ' . __FUNCTION__ . ' for your self.');
	} // END - if

	// Secure the string
	$_SERVER['PHP_SELF'] = secureString($_SERVER['PHP_SELF']);

	// Split it up into path and filename
	$phpSelfDirectory = dirname($_SERVER['PHP_SELF']);
	$phpSelfFile      = basename($_SERVER['PHP_SELF']);

	// Check for a .php inside the $phpSelfDirectory...
	while (strpos($phpSelfDirectory, '.php') !== false) {
		// Correct the dirname
		$phpSelfDirectory = substr($phpSelfDirectory, 0, (strpos($phpSelfDirectory, '.php') + 4));
		// Rewrite filename...
		$phpSelfFile = basename($phpSelfDirectory);
		// ... and dirname
		$phpSelfDirectory = dirname($phpSelfDirectory);
	} // END - while

	// Put both together again and let's pray it is secured now...
	$_SERVER['PHP_SELF'] = $phpSelfDirectory . '/' . $phpSelfFile;

	// Did run...
	$GLOBALS['php_self_secured'] = true;

	// Remove uneccessary variables
	unset($phpSelfDirectory);
	unset($phpSelfFile);
}

/**
 * Detects caching in PHP
 *
 * @return	void
 */
function detectPhpCaching () {
	// Activate caching or transparent compressing when it is not already done
	if (phpversion() >= '4.0.4pl1' && (strstr(getenv('HTTP_USER_AGENT'),'compatible') || (strstr(getenv('HTTP_USER_AGENT'), 'Mozilla')))) {
		if ((extension_loaded('zlib')) && (function_exists('ob_start'))) {
			// Start caching
			$GLOBALS['php_caching'] = 'on';
			ob_start();
		} else {
			// Extension not loaded or required function is missing
			$GLOBALS['php_caching'] = '404';
		}
	} else {
		// Old PHP version
		$GLOBALS['php_caching'] = 'old';
	}
}

// Runtime/GPC quoting is off now...
ini_set('magic_quotes_runtime', false);
ini_set('magic_quotes_gpc', false); // This may not work on some systems

// No compatibility with Zend Engine 1, else an error like 'Implicit cloning'
// will be produced.
if (phpversion() >= '5.0') {
	ini_set('zend.ze1_compatibility_mode', 'Off');
} // END - if

// Check if important arrays are found and define them if missing
if (!isset($_SERVER)) {
	global $_SERVER;
	$_SERVER = $GLOBALS['_SERVER'];
} // END - if

if (!isset($_GET)) {
	global $_GET;
	$_GET = $GLOBALS['_GET'];
} // END - if

if (!isset($_POST)) {
	global $_POST;
	$_POST = $GLOBALS['_POST'];
} // END - if

// Generate arrays which holds the relevante chars to replace
$GLOBALS['security_chars'] = array(
	// The chars we are looking for...
	'from' => array('/', '.', "'", '$', '(', ')', '{--', '--}', '{?', '?}', '%', ';', '[', ']', ':', '--', "\\"),
	// ... and we will replace to.
	'to'   => array(
		'{SLASH}',
		'{DOT}',
		'{QUOT}',
		'{DOLLAR}',
		'{OPEN_ANCHOR}',
		'{CLOSE_ANCHOR}',
		'{OPEN_TEMPLATE}',
		'{CLOSE_TEMPLATE}',
		'{OPEN_CONFIG}',
		'{CLOSE_CONFIG}',
		'{PER}',
		'{SEMI}',
		'{OPEN_INDEX}',
		'{CLOSE_INDEX}',
		'{DBL_DOT}',
		'{COMMENT}',
		'{BACKSLASH}'
	),
);

// Characters allowed in URLs
//
// Note: Do not replace 'to' with 'from' and vise-versa! When you do this all booked URLs will be
//       rejected because of the {SLASH}, {DOT} and all below listed items inside the URL.
$GLOBALS['url_chars'] = array(
	// Search for these secured characters
	'to'   => array('{SLASH}', '{DOT}', '{PER}', '{DBL_DOT}', '{COMMENT}'),
	// Replace with these characters
	'from' => array('/', '.', '%', ':', '--')
);

// Overworked security part:
if (is_array($_GET)) {
	foreach ($_GET as $seckey => $secvalue) {
		if (is_array($secvalue)) {
			// Throw arrays away...
			unset($_GET[$seckey]);
		} else {
			// Only variables are allowed (non-array) but we secure them all!
			$_GET[$seckey] = str_replace($GLOBALS['security_chars']['from'], $GLOBALS['security_chars']['to'], $_GET[$seckey]);

			// Strip all other out
			$_GET[$seckey] = secureString($_GET[$seckey]);
		}
	} // END - foreach
} // END - if

// Secure also $_POST data (only simple, no replace)
if (is_array($_POST)) {
	// Secure only simple data
	foreach ($_POST as $seckey => $secvalue) {
		// Is it an array?
		if (!is_array($secvalue)) {
			// Strip all other out
			$_POST[$seckey] = secureString($_POST[$seckey]);
		} // END - if
	} // END - foreach
} // END - if

// Detect PHP caching
detectPhpCaching();

// At last secure the $_SERVER['PHP_SELF'] element
securePhpSelf();

// Security system loaded...
define('__SECURITY', 1);

// [EOF]
?>
